/*
 *  IBM Confidential
 *  OCO Source Materials
 *
 *  (C) Copyright IBM Corp. 2005
 *
 *  The source code for this program is not published or otherwise divested
 *  of its trade secrets, irrespective of what has been deposited with the
 *  U.S. Copyright Office.
 *
 *
 */
package com.ibm.wtci.samples.axis.source;

import com.ibm.wtci.samples.axiswsn13.wsa.AttributedURIType;
import com.ibm.wtci.samples.axiswsn13.wsa.EndpointReferenceType;
import com.ibm.wtci.samples.axiswsn13.wsn.NotificationConsumer;
import com.ibm.wtci.samples.axiswsn13.wsn.NotificationConsumerService;
import com.ibm.wtci.samples.axiswsn13.wsn.NotificationConsumerServiceLocator;
import com.ibm.wtci.samples.axiswsn13.wsn.NotificationMessageHolderType;
import com.ibm.wtci.samples.axiswsn13.wsn.NotificationMessageHolderTypeMessage;
import com.ibm.wtci.samples.axiswsn13.wsn.Notify;
import com.ibm.wtci.samples.axiswsn13.wsn.TopicExpressionType;

import org.apache.axis.encoding.TypeMapping;
import org.apache.axis.encoding.TypeMappingRegistry;
import org.apache.axis.message.MessageElement;
import org.apache.axis.types.URI;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.hyades.logging.events.cbe.CommonBaseEvent;
import org.eclipse.hyades.logging.events.cbe.EventFactory;
import org.eclipse.hyades.logging.events.cbe.EventFactoryFactory;
import org.eclipse.hyades.logging.events.cbe.EventPackage;
import org.eclipse.hyades.logging.events.cbe.Situation;
import org.eclipse.hyades.logging.events.cbe.util.EventFormatter;

import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.io.File;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;

/**
 * Sample code for a WS-Notification 1.3 event producer.
 * 
 * @version $Header: /cvsroot/mainstream/wtci/code/samples/wsnt-axis/src/com/ibm/wtci/samples/axis/source/EventSourceWsn13.java,v 1.2 2006/05/07 11:20:44 rradhika Exp $ $Revision: 1.2 $
 * 
 * @since tec_3.9.0_fo1
 */
public class EventSourceWsn13 {

    /**
     * Main method for the event source.
     * 
     * @param args
     *            arguments passed from the command line
     */
    public static void main(String args[]) {
        EventSourceWsn13 source = new EventSourceWsn13();
        try {
            source.sendEvent();
            System.out.println("Notification sent");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Sends a notification message.
     * <p>
     * The request is compliant with the <code>NotificationConsumer</code>
     * port defined in the WS-Notification 1.3 specification.
     * 
     * @throws Exception
     *             if the notification cannot be sent for whatever reason.
     */
    private void sendEvent() throws Exception {
        //The first step is creating an event
        CommonBaseEvent[] events = createEvents();

        //Creating the Notification message encapsulating the event
        Notify notification = createNotifyMessage(events);

        //Obtaining the address of the NotificationConsumerService webservice
        String endpoint = System.getProperty("service.address");
        if (endpoint == null) {
            //If no address was specified, the webservice is assumed to be
            // runnning in the localhost at port 8080 for axis
            endpoint = "http://localhost:8080/axis/services/NotificationConsumer10Soap";
        }

        //Creating an URL object for the address obtained
        URL serviceURL = new URL(endpoint);

        //Creating an instance of NotificationConsumerServiceLocator
        NotificationConsumerService notifierSvc = new NotificationConsumerServiceLocator();

        //Using the NotificationConsumerService to obtain
        // NotificationConsumer object for the service
        NotificationConsumer notifier = notifierSvc.getNotificationConsumer13Soap(serviceURL);

        //Registering an instance of CbeSerializer object for serializing
        // objects of
        //type CommonBaseEvent to be sent over the wire
        TypeMappingRegistry reg = (TypeMappingRegistry) notifierSvc.getTypeMappingRegistry();
        TypeMapping tm = (TypeMapping) reg.getTypeMapping(endpoint);
        QName qn = new QName("http://www.ibm.com/AC/commonbaseevent1_0_1",
                "CommonBaseEvent");
        tm.register(CommonBaseEvent.class, qn, new CbeSerializerFactory(), null);

        //Sending the notification
        notifier.notify(notification);
    }

    /**
     * Returns an array of CommonBaseEvent instances.
     * <p>
     * This method looks at the system property "cbe.file" and attempts to read
     * the events from the file indicated in the property. If the property is
     * not set, an array with one hardcoded event is returned.
     * 
     * @return an array of event instances
     * 
     * @throws Exception
     *             if the events cannot be created for whatever reason
     */
    private CommonBaseEvent[] createEvents() throws Exception {
        CommonBaseEvent[] events = null;

        String cbeFile = System.getProperty("cbe.file");
        if (cbeFile != null) {
            events = EventFormatter.eventsFromCanonicalXMLDoc(new File(cbeFile));
        } else {
            // The first step is accessing the event factory
            EventFactory eventFactory = EventFactoryFactory.createEventFactory();

            // Creating an event with an extension name.
            CommonBaseEvent event = eventFactory.createCommonBaseEvent("EVENT");
            event.setCreationTimeAsLong(System.currentTimeMillis());

            // Setting the mandatory situation description for the event.
            Situation situation = eventFactory.createSituation();
            situation.setCategoryName(Situation.REPORT_SITUATION_CATEGORY);
            situation.setReportSituation("INTERNAL", "Succeeded");
            event.setSituation(situation);

            // Setting the mandatory component identification for the
            // event source
            event.setSourceComponentId("Event Source",
                "source.EventSource",
                "createEvent()",
                "http://www.ibm.com/namespaces/autonomic/Tivoli/Samples",
                "Sample",
                "unknown",
                "Hostname");

            // Setting Common Base Event version
            event.setVersion("1.0.1");
            // Setting optional fields
            event.setSeverity((short) 10);
            event.setMsg("Common Event Infrastructure Tutorial");
            event.validate();

            events = new CommonBaseEvent[] { event };
        }

        return events;
    }

    /**
     * Creates a WS-Notification message encapsulating a events of type
     * CommonBaseEvent.
     * 
     * @param events
     *            array of events to be carried inside the message.
     * 
     * @return an array of notification messages containg the events to be sent.
     * 
     * @throws Exception
     *             if the notification cannot be created for whatever reason.
     */
    private Notify createNotifyMessage(CommonBaseEvent[] events)
            throws Exception {

        NotificationMessageHolderType[] notificationMessages = new NotificationMessageHolderType[events.length];
        for (int i = 0; i < events.length; i++) {

            // Create topic
            TopicExpressionType topicExpressionType = new TopicExpressionType();
            MessageElement[] topicDefinition = new MessageElement[1];
            topicDefinition[0] = new MessageElement();
            topicDefinition[0].setName("All");
            topicDefinition[0].setAttribute("dialect", "tec");
            topicDefinition[0].setNamespaceURI("http://www.ibm.com/events/wsn13-wtci-1.0.0");
            topicExpressionType.set_any(topicDefinition);

            // Create endpoint reference
            EndpointReferenceType endpointReference = new EndpointReferenceType();
            AttributedURIType endpointURI = new com.ibm.wtci.samples.axiswsn13.wsa.AttributedURIType();
            URI uriValue = new URI("scheme", "your.event.source.address");
            endpointURI.set_value(uriValue);
            endpointReference.setAddress(endpointURI);

            // Create notification message
            NotificationMessageHolderTypeMessage notificationMessage = new NotificationMessageHolderTypeMessage();
            MessageElement cbeElement = createMessageElement(events[i]);
            cbeElement.setAttribute("xmlns",
                "http://www.ibm.com/AC/commonbaseevent1_0_1");
            MessageElement[] notificationMessageElements = { cbeElement };
            notificationMessage.set_any(notificationMessageElements);

            // Create notification message value
            NotificationMessageHolderType payload = new NotificationMessageHolderType();
            payload.setTopic(topicExpressionType);
            payload.setMessage(notificationMessage);
            payload.setProducerReference(endpointReference);
            notificationMessages[i] = payload;
        }

        Notify result = new Notify();
        result.setNotificationMessage(notificationMessages);

        return result;
    }

    /**
     * Required to set the xsi:type in the <code>situationType</code> elements
     * of a CBE.
     */
    private static final EAttribute categoryName = EventPackage.eINSTANCE.getSituation_CategoryName();

    /**
     * Transposes the data from an EMF-based object into an AXIS MessageElement
     * object.
     * 
     * @param source
     *            the EMF object
     * 
     * @return the message element containing the EMF data.
     * 
     * @throws SOAPException
     *             if the data cannot be converted for whatever reason.
     */
    private MessageElement createMessageElement(EObject source)
            throws SOAPException {
        MessageElement result = new MessageElement();

        EReference containmentFeature = source.eContainmentFeature();
        String elementName = containmentFeature == null ? source.eClass()
            .getName() : containmentFeature.getName();
        result.setName(elementName);

        List attributes = source.eClass().getEAllAttributes();
        int contentSize = attributes.size();
        for (int i = 0; i < contentSize; i++) {
            EAttribute attribute = (EAttribute) attributes.get(i);

            Object sourceValue = source.eGet(attribute);
            boolean sourceIsSet = source.eIsSet(attribute);

            if (sourceIsSet && sourceValue != null) {
                if (attribute.isMany()) {
                    List sourceValueList = (List) sourceValue;
                    for (Iterator j = sourceValueList.iterator(); j.hasNext();) {
                        Object listItem = (Object) j.next();
                        SOAPElement childElement = result.addChildElement(attribute.getName());
                        childElement.addTextNode(listItem.toString());
                    }
                } else {
                    result.addAttribute(null,
                        attribute.getName(),
                        sourceValue.toString());
                }
            } // if source value not null
        } // for attributes

        List containments = source.eClass().getEAllContainments();
        contentSize = containments.size();
        for (int i = 0; i < contentSize; i++) {
            EStructuralFeature containment = (EStructuralFeature) containments.get(i);

            Object sourceValue = source.eGet(containment);
            boolean sourceIsSet = source.eIsSet(containment);

            if (sourceIsSet && sourceValue != null) {
                if (containment.isMany()) {
                    for (Iterator j = ((EList) sourceValue).iterator(); j.hasNext();) {
                        EObject listElement = (EObject) j.next();
                        MessageElement childElement = createMessageElement(listElement);
                        result.addChildElement(childElement);
                    }
                } else {
                    EObject childEObject = (EObject) sourceValue;
                    MessageElement childElement = createMessageElement(childEObject);
                    result.addChildElement(childElement);
                    if ("situationType".equals(childElement.getLocalName())) {
                        childElement.addAttribute("xsi",
                            "http://www.w3.org/2001/XMLSchema-instance",
                            "type",
                            source.eGet(categoryName).toString());
                    }
                }
            } // if source value not null
        } // for features

        return result;
    }
}
